
import "Turbine";
import "Turbine.UI";
import "Turbine.UI.Lotro";

-- An Error on the following line indicates Turbine API has not been installed yet!
-- In this case read the installation instructions first.
EffectDisplay = class( Turbine.UI.Control );

local baseImageID = 0x41007E35;

function EffectDisplay:Constructor()
	Turbine.UI.Control.Constructor( self );

	self.display = Turbine.UI.Lotro.EffectDisplay();
	self.display:SetParent( self );

	self.threshold=1;

	self.elapsedOverlay = Turbine.UI.Control();
	self.elapsedOverlay:SetParent( self );
	self.elapsedOverlay:SetStretchMode( 1 );
	self.elapsedOverlay:SetMouseVisible( false );
	self.elapsedOverlay:SetZOrder( 10 );
	self.elapsedOverlay:SetOpacity( 0.7 );
	self.elapsedOverlay:SetBackColor( Turbine.UI.Color( 1, 0, 0, 0 ) );
	self.elapsedOverlay:SetBackColorBlendMode( Turbine.UI.BlendMode.Multiply );
end

function EffectDisplay:SetMouseVisible( value )
	Turbine.UI.Control.SetMouseVisible( self, value );
	self.display:SetVisible(true);
end

function EffectDisplay:SetEffect( value,  timeLabel, alertLabel, instruction, expireMode )
	self.display:SetEffect( value );
	self.currentEffect = value;
	self.timeCaption = timeLabel;
	self.alertCaption = alertLabel;
	self.instruction = instruction;
	self.expireMode = expireMode;
	
	if ( value ~= nil ) then
		self.startTime = value:GetStartTime();
		self.duration = value:GetDuration();
		self.effectEndTime = ( self.startTime + self.duration );

		if ( Turbine.Engine.GetGameTime() < self.effectEndTime ) then
			self:SetWantsUpdates( true );
			self:SetVisible(true);
		end
	end
end

function EffectDisplay:SetPriorityThreshold( value )
	self.threshold=value;
end

function EffectDisplay:GetPriorityThreshold( )
	return (self.threshold);
end

function EffectDisplay:CheckPriority( newInstruction )
	if self.instruction==nil then
		return (newInstruction:GetPriority()>=self.threshold);
	end 
	return (newInstruction:GetPriority()>=self.threshold and
		newInstruction:GetPriority()>self.instruction:GetPriority());
end

function EffectDisplay:RemoveEffect( effect )
	if self.currentEffect ~= nil then
		if self.currentEffect:GetID() == effect:GetID() then
				self:SetWantsUpdates( false );
				self:SetVisible(false);
				self.timeCaption:SetText(" ");
				self.alertCaption:SetText(" ");
				self.instruction = nil;
				return true;
		end
	end
	return false;
end

function EffectDisplay:Update( args )
	local gameTime = Turbine.Engine.GetGameTime();
	local elapsedTime = gameTime - self.startTime;
	local percentComplete = elapsedTime / self.duration;

	local imageId = baseImageID + math.floor( 60 * percentComplete );

	self.elapsedOverlay:SetStretchMode( 3 );
	self.elapsedOverlay:SetPosition( 1, 1 );
	self.elapsedOverlay:SetSize( 32, 32 );
	self.elapsedOverlay:SetBackground( imageId );

	local width, height = self:GetSize();
	self.elapsedOverlay:SetStretchMode( 1 );
	self.elapsedOverlay:SetSize( width - 2, height - 2 );

	local timeLeft = self.effectEndTime - gameTime;
	if self.expireMode then
		if timeLeft < 99 and timeLeft >= 0 then
			self.timeCaption:SetText(""..math.floor(timeLeft).."s");
		else
			self.timeCaption:SetText(" ");
			self.alertCaption:SetText(" ");
		end
	elseif timeLeft >= 1 then
		if timeLeft <= 3600 then
			self.timeCaption:SetText(""..math.floor(timeLeft).."s");
		end
	else
		self.timeCaption:SetText(" ");
		self.alertCaption:SetText(" ");
	end
	
	if self.expireMode then
		self:SetVisible(timeLeft<99 and timeLeft>=0);
	elseif ( gameTime > self.effectEndTime + 1 ) then
		self:SetWantsUpdates( false );
		self:SetVisible(false);
	end
end

function EffectDisplay:SizeChanged( args )
	local width, height = self:GetSize();

	self.display:SetSize( width - 2, height - 2 );
	self.elapsedOverlay:SetSize( width - 2, height - 2 );
end
