
import "Turbine";
import "Turbine.UI";
import "Turbine.UI.Lotro";

import "OozStuff.Carebear.Resource";
import "OozStuff.Carebear.NamedEffectInstructions";
import "OozStuff.Carebear.DropDown";
import "OozStuff.OozUI.Border";
import "OozStuff.OozUI.IconLabel";


EffectManager = class( Turbine.UI.Lotro.Window );

local baseImageID = 0x41007E35;

function EffectManager:Constructor(language)
	Turbine.UI.Lotro.Window.Constructor( self );
	local initWidth=500;
	local initHeight=425;
	self:SetSize(initWidth,initHeight);

	self.language = language;
	self:SetText(Resource[self.language][9]);
	self.activeEffectInstructions = {};
	self.inactiveEffectInstructions = {};
	self.uncategorizedEffectInstructions = {};
	self.activeEffectInstructionsDirty = true;
	self.inactiveEffectInstructionsDirty = true;
	self.uncategorizedEffectInstructionsDirty = true;

	self.currentInstruction = nil;
	
	-- Upper left part: Dropdown with Listbox
	
	self.ddValues = {Resource[language][10], Resource[language][11], Resource[language][12]};
	self.activeList = 1;
	self.ddList = OozStuff.Carebear.DropDown.Create(self.ddValues, self.ddValues[self.activeList]);
	self.ddList:SetParent(self);
	self.ddList:ApplyWidth(300); -- To set the width if you want a smaller/bigger menu.
	self.ddList:SetPosition(20,36);
	self.ddList:SetVisible(true);
	self.ddList.ItemChanged=function()
		for i=1,#self.ddValues do
			if self.ddValues[i]==self.ddList:GetText() then
				self.activeList = i;
				self.lListHelp:SetText(Resource[self.language][15+self.activeList]);
				self.lcPriorityThreshold:SetVisible(self.activeList==2);
				self.ddPriorityThreshold:SetVisible(self.activeList==2);
			end
		end
		self:ChangeListValues(false);
	end
	
    self.scrollBar = Turbine.UI.Lotro.ScrollBar();
    self.scrollBar:SetZOrder(51);
    self.scrollBar:SetOrientation(Turbine.UI.Orientation.Vertical);
    self.scrollBar:SetParent(self);
	
	local fontFace=Turbine.UI.Lotro.Font.Verdana12;
	local lineHeight = 14;
	
    -- list to contain the drop down items
	self.fsize = 32;
    self.listBox = Turbine.UI.ListBox();
    self.listBox:SetParent(self);
    self.listBox:SetOrientation(Turbine.UI.Orientation.Horizontal);
    self.listBox:SetVerticalScrollBar(self.scrollBar);
    self.listBox:SetMaxItemsPerLine(1);
    self.listBox:SetMouseVisible(false);
    self.listBox:SetPosition(21,37+self.ddList:GetHeight());
	self.listBox:SetSize(300-2,self.fsize*5);
    self.listBox:SetZOrder(50);

	local borderFColor=Turbine.UI.Color(.5,.45,.25);
	local borderBColor=Turbine.UI.Color(.15,.15,.15);
    self.listBoxBorder = OozStuff.OozUI.Border();
    self.listBoxBorder:SetParent(self);
    self.listBoxBorder:Decorate(self.listBox); --, borderFColor, borderBColor);
	
    self.scrollBar:SetSize(10, self.listBoxBorder:GetHeight());
    self.scrollBar:SetPosition(self.listBoxBorder:GetLeft()-10, self.listBoxBorder:GetTop());
	
	-- Upper right part: List Management (Buttons) and Priority 

    self.lListHelp = Turbine.UI.Label();
	self.lListHelp:SetParent(self);
	self.lListHelp:SetText(Resource[self.language][15+self.activeList]);
	self.lListHelp:SetPosition( 330, 40 );
	self.lListHelp:SetWidth(150);
	self.lListHelp:SetHeight(100);
	self.lListHelp:SetForeColor(borderFColor);

	local highlightColor=Turbine.UI.Color(.2,.2,.6);
	local buttonColor=Turbine.UI.Color(.6,.6,.2);
	
	self.ToActiveButton = Turbine.UI.Lotro.Button();
	self.ToActiveButton:SetParent(self);
	self.ToActiveButton:SetPosition( 330, 90 );
	self.ToActiveButton:SetWidth( 150 );
	self.ToActiveButton:SetText( Resource[language][13] );
	self.ToActiveButton:SetEnabled(false);
	self.ToActiveButton.MouseClick = function( sender, args )
		self:MoveSelectedItemToList(2);
	end

	self.ToIgnoredButton = Turbine.UI.Lotro.Button();
	self.ToIgnoredButton:SetParent(self);
	self.ToIgnoredButton:SetPosition( 330, 120 );
	self.ToIgnoredButton:SetWidth( 150 );
	self.ToIgnoredButton:SetText( Resource[language][14] );
	self.ToIgnoredButton:SetEnabled(false);
	self.ToIgnoredButton.MouseClick = function( sender, args )
		self:MoveSelectedItemToList(3);
	end
	
	self.ToDeletedButton = Turbine.UI.Lotro.Button();
	self.ToDeletedButton:SetParent(self);
	self.ToDeletedButton:SetPosition( 330, 150 );
	self.ToDeletedButton:SetWidth( 150 );
	self.ToDeletedButton:SetText( "Delete >" );
	self.ToDeletedButton:SetVisible(verbose>=2);
	self.ToDeletedButton.MouseClick = function( sender, args )
		self:MoveSelectedItemToList(0);
	end
	
    self.pBox = Turbine.UI.ListBox();
    self.pBox:SetParent(self);
    self.pBox:SetOrientation(Turbine.UI.Orientation.Horizontal);
    self.pBox:SetMaxItemsPerLine(1);
    self.pBox:SetPosition( 330, 185);
	self.pBox:SetSize(150, 30);
    self.pBox:SetZOrder(50);
    self.lcPriorityThreshold = Turbine.UI.Label();
	self.lcPriorityThreshold:SetText(Resource[self.language][50]);
	self.lcPriorityThreshold:SetWidth(150);
	self.lcPriorityThreshold:SetHeight(14);
	self.lcPriorityThreshold:SetVisible(false);
	self.pBox:AddItem(self.lcPriorityThreshold);
	self.priorityValues = {Resource[self.language][42],Resource[self.language][43],Resource[self.language][44],
			Resource[self.language][45]};
	self.ddPriorityThreshold = DropDown.Create(self.priorityValues, self.priorityValues[1]);
	self.ddPriorityThreshold:ApplyWidth(150);
	self.ddPriorityThreshold:SetVisible(false);
	self.ddPriorityThreshold.ItemChanged=function()
		if 	self.currentInstruction~=nil then
			for i=1,#self.priorityValues do
				if self.priorityValues[i]==self.ddPriorityThreshold:GetText() then
					mainWindow.effectDisplay:SetPriorityThreshold(i);
				end
			end
		end
	end
	self.pBox:AddItem(self.ddPriorityThreshold);
	
	-- Lower part: Effect Instruction Details

    self.iBox = Turbine.UI.ListBox();
    self.iBox:SetParent(self);
    self.iBox:SetOrientation(Turbine.UI.Orientation.Horizontal);
    self.iBox:SetMaxItemsPerLine(2);
    self.iBox:SetPosition(self.listBox:GetLeft(),self.listBox:GetTop()+self.listBox:GetHeight()+10);
    self.iBox:SetPosition(20,225);
	self.iBox:SetSize(initWidth-self.iBox:GetLeft()-20,initHeight-self.iBox:GetTop()-20);
    self.iBox:SetZOrder(50);
	
    self.lcEffectName = Turbine.UI.Label();
	self.lcEffectName:SetText(Resource[self.language][19]);
	self.lcEffectName:SetWidth(100);
	self.lcEffectName:SetHeight(lineHeight);
	self.iBox:AddItem(self.lcEffectName);
    self.lEffectName = Turbine.UI.Label();
	self.lEffectName:SetText("");
	self.lEffectName:SetWidth(initWidth-150);
	self.lEffectName:SetHeight(lineHeight);
	self.iBox:AddItem(self.lEffectName);

    self.lcEffectInfo = Turbine.UI.Label();
	self.lcEffectInfo:SetText(Resource[self.language][20]);
	self.lcEffectInfo:SetWidth(100);
	self.lcEffectInfo:SetHeight(lineHeight);
	self.iBox:AddItem(self.lcEffectInfo);
    self.lEffectInfo = Turbine.UI.Label();
	self.lEffectInfo:SetText("");
	self.lEffectInfo:SetWidth(initWidth-150);
	self.lEffectInfo:SetHeight(lineHeight);
	self.iBox:AddItem(self.lEffectInfo);

    self.lcEffectDesc = Turbine.UI.Label();
	self.lcEffectDesc:SetText(Resource[self.language][21]);
	self.lcEffectDesc:SetWidth(100);
	self.lcEffectDesc:SetHeight(lineHeight*3);
	self.iBox:AddItem(self.lcEffectDesc);
    self.lEffectDesc = Turbine.UI.Label();
	self.lEffectDesc:SetText("");
	self.lEffectDesc:SetWidth(initWidth-150);
	self.lEffectDesc:SetHeight(lineHeight*3);
	self.iBox:AddItem(self.lEffectDesc);

    self.lcEffectCureAction = Turbine.UI.Label();
	self.lcEffectCureAction:SetText(Resource[self.language][22]);
	self.lcEffectCureAction:SetWidth(100);
	self.lcEffectCureAction:SetHeight(22);
	self.iBox:AddItem(self.lcEffectCureAction);
	self.cureValues = {Resource[self.language][23],Resource[self.language][24],Resource[self.language][25],Resource[self.language][57]};
	self.ddEffectCureAction = DropDown.Create(self.cureValues, self.cureValues[1]);
	self.ddEffectCureAction:ApplyWidth(initWidth-150);
	self.ddEffectCureAction.ItemChanged=function()
		if 	self.currentInstruction~=nil then
			for i=1,#self.cureValues do
				if self.cureValues[i]==self.ddEffectCureAction:GetText() then
					self.currentInstruction:SetActionCure(i);
					if self.activeList==1 then
						self.uncategorizedEffectInstructionsDirty = true;
					elseif self.activeList==2 then
						self.activeEffectInstructionsDirty = true;
					elseif self.activeList==3 then
						self.inactiveEffectInstructionsDirty = true;
					end
					break;
				end
			end
		end
	end
	self.iBox:AddItem(self.ddEffectCureAction);

    self.lcEffectMoveAction = Turbine.UI.Label();
	self.lcEffectMoveAction:SetText(Resource[self.language][26]);
	self.lcEffectMoveAction:SetWidth(100);
	self.lcEffectMoveAction:SetHeight(22);
	self.iBox:AddItem(self.lcEffectMoveAction);
	self.moveValues = {Resource[self.language][27],Resource[self.language][28],Resource[self.language][29],
			Resource[self.language][30],Resource[self.language][31]};
	self.ddEffectMoveAction = DropDown.Create(self.moveValues, self.moveValues[1]);
	self.ddEffectMoveAction:ApplyWidth(initWidth-150);
	self.ddEffectMoveAction.ItemChanged=function()
		if 	self.currentInstruction~=nil then
			for i=1,#self.moveValues do
				if self.moveValues[i]==self.ddEffectMoveAction:GetText() then
					self.currentInstruction:SetActionMove(i);
					if self.activeList==1 then
						self.uncategorizedEffectInstructionsDirty = true;
					elseif self.activeList==2 then
						self.activeEffectInstructionsDirty = true;
					elseif self.activeList==3 then
						self.inactiveEffectInstructionsDirty = true;
					end
					break;
				end
			end
		end
	end
	self.iBox:AddItem(self.ddEffectMoveAction);

    self.lcEffectPriority = Turbine.UI.Label();
	self.lcEffectPriority:SetText(Resource[self.language][41]);
	self.lcEffectPriority:SetWidth(100);
	self.lcEffectPriority:SetHeight(22);
	self.iBox:AddItem(self.lcEffectPriority);
	self.ddEffectPriority = DropDown.Create(self.priorityValues, self.priorityValues[1]);
	self.ddEffectPriority:ApplyWidth(initWidth-150);
	self.ddEffectPriority.ItemChanged=function()
		if 	self.currentInstruction~=nil then
			for i=1,#self.priorityValues do
				if self.priorityValues[i]==self.ddEffectPriority:GetText() then
					self.currentInstruction:SetPriority(i);
					if self.activeList==1 then
						self.uncategorizedEffectInstructionsDirty = true;
					elseif self.activeList==2 then
						self.activeEffectInstructionsDirty = true;
					elseif self.activeList==3 then
						self.inactiveEffectInstructionsDirty = true;
					end
					break;
				end
			end
		end
	end
	self.iBox:AddItem(self.ddEffectPriority);
	
    self.lcEffectYellAction = Turbine.UI.Label();
	self.lcEffectYellAction:SetText(Resource[self.language][32]);
	self.lcEffectYellAction:SetWidth(100);
	self.lcEffectYellAction:SetHeight(22);
	self.iBox:AddItem(self.lcEffectYellAction);
    self.cbEffectYellAction = Turbine.UI.Lotro.CheckBox();
	self.cbEffectYellAction:SetText(Resource[self.language][33]);
	self.cbEffectYellAction:SetWidth(initWidth-150);
	self.cbEffectYellAction:SetHeight(18);
	self.cbEffectYellAction:SetFont(fontFace);
	self.cbEffectYellAction.CheckedChanged = function( sender, args )
		if 	self.currentInstruction~=nil then
			self.currentInstruction:SetYellAction(self.cbEffectYellAction:IsChecked());
			if self.activeList==1 then
				self.uncategorizedEffectInstructionsDirty = true;
			elseif self.activeList==2 then
				self.activeEffectInstructionsDirty = true;
			elseif self.activeList==3 then
				self.inactiveEffectInstructionsDirty = true;
			end
		end
	end
	self.iBox:AddItem(self.cbEffectYellAction);

    self.lcEffectComment = Turbine.UI.Label();
	self.lcEffectComment:SetText(Resource[self.language][34]);
	self.lcEffectComment:SetWidth(100);
	self.lcEffectComment:SetHeight(lineHeight);
	self.iBox:AddItem(self.lcEffectComment);
    self.tEffectComment = Turbine.UI.Lotro.TextBox();
	self.tEffectComment:SetText("");
	self.tEffectComment:SetWidth(initWidth-150);
	self.tEffectComment:SetHeight(18);
	self.tEffectComment:SetFont(fontFace);
	self.tEffectComment.FocusLost = function( sender, args )
		if 	self.currentInstruction~=nil then
			self.currentInstruction:SetComment(self.tEffectComment:GetText());
			if self.activeList==1 then
				self.uncategorizedEffectInstructionsDirty = true;
			elseif self.activeList==2 then
				self.activeEffectInstructionsDirty = true;
			elseif self.activeList==3 then
				self.inactiveEffectInstructionsDirty = true;
			end
		end
	end
	self.iBox:AddItem(self.tEffectComment);

	
	-- Event Handling:

	self.listBox.SelectedIndexChanged=function(sender, args)
		local index = self.listBox:GetSelectedIndex();
		local itemCount = self.listBox:GetItemCount();
		if index<1 or index>itemCount then
			self.ToIgnoredButton:SetEnabled(false);
			self.ToActiveButton:SetEnabled(false);
			return;
		end

		local instructions = { };
		if self.activeList==1 then
			instructions=self.uncategorizedEffectInstructions;
		elseif self.activeList==2 then
			instructions=self.activeEffectInstructions;
		elseif self.activeList==3 then
			instructions=self.inactiveEffectInstructions;
		end

		self.currentInstruction = nil;
		for i=1,itemCount do
			local itemLabel = self.listBox:GetItem(i);
			if i==index then
				itemLabel:SetForeColor(backColor);
				itemLabel:SetBackColor(fontColor);
				
				self.currentInstruction = instructions[index];
				
				self.lEffectName:SetText(self.currentInstruction:GetName(self.language));
				self.lEffectDesc:SetText(self.currentInstruction:GetDescription(self.language));
				local infoText = "";
				if self.currentInstruction:GetDamageType()==NEM_M_FLAG_DISEASE then
					infoText=infoText..Resource[self.language][35];
				elseif self.currentInstruction:GetDamageType()==NEM_M_FLAG_WOUND then
					infoText=infoText..Resource[self.language][36];
				elseif self.currentInstruction:GetDamageType()==NEM_M_FLAG_FEAR then
					infoText=infoText..Resource[self.language][37];
				elseif self.currentInstruction:GetDamageType()==NEM_M_FLAG_POISON then
					infoText=infoText..Resource[self.language][38];
				end
				if self.currentInstruction:IsCurable() then
					infoText=infoText.." "..Resource[self.language][39];
				end
				if self.currentInstruction:IsCurable() then
					infoText=infoText.." "..Resource[self.language][40];
				end
				self.lEffectInfo:SetText(infoText);
				self.ddEffectPriority:SetLabel(self.priorityValues[self.currentInstruction:GetPriority()]);
				self.ddEffectCureAction:SetLabel(self.cureValues[self.currentInstruction:GetActionCure()]);
				self.ddEffectMoveAction:SetLabel(self.moveValues[self.currentInstruction:GetActionMove()]);
				self.cbEffectYellAction:SetChecked(self.currentInstruction:HasYellAction());
				
				self.tEffectComment:SetText(self.currentInstruction:GetComment());
			else
				itemLabel:SetForeColor(fontColor);
				itemLabel:SetBackColor(backColor);
			end
		end
	end
end

function EffectManager:MoveSelectedItemToList(number)
	local index = self.listBox:GetSelectedIndex();
	if index<1 then
		return;
	end
	local instruction = nil;

	local item = self.listBox:GetItem(index);
	self.listBox:RemoveItem(item);

	if self.activeList==1 then
		instruction = self.uncategorizedEffectInstructions[index];
		table.remove(self.uncategorizedEffectInstructions,index);
		self.uncategorizedEffectInstructionsDirty = true;
		if index<=self.listBox:GetItemCount() then
			self.listBox:SetSelectedIndex(index);
		elseif self.listBox:GetItemCount()>0 then
			self.listBox:SetSelectedIndex(self.listBox:GetItemCount());
		end
	elseif self.activeList==2 then
		instruction = self.activeEffectInstructions[index];
		table.remove(self.activeEffectInstructions,index);
		self.activeEffectInstructionsDirty = true;
		if index<=self.listBox:GetItemCount() then
			self.listBox:SetSelectedIndex(index);
		elseif self.listBox:GetItemCount()>0 then
			self.listBox:SetSelectedIndex(self.listBox:GetItemCount());
		end
	elseif self.activeList==3 then
		instruction = self.inactiveEffectInstructions[index];
		table.remove(self.inactiveEffectInstructions,index);
		self.inactiveEffectInstructionsDirty = true;
		if index<=self.listBox:GetItemCount() then
			self.listBox:SetSelectedIndex(index);
		elseif self.listBox:GetItemCount()>0 then
			self.listBox:SetSelectedIndex(self.listBox:GetItemCount());
		end
	end
	
	if number==1 then
		self.uncategorizedEffectInstructions[#self.uncategorizedEffectInstructions+1]=instruction;
		self.uncategorizedEffectInstructionsDirty = true;
	elseif number==2 then
		self.activeEffectInstructions[#self.activeEffectInstructions+1]=instruction;
		self.activeEffectInstructionsDirty = true;
	elseif number==3 then
		self.inactiveEffectInstructions[#self.inactiveEffectInstructions+1]=instruction;
		self.inactiveEffectInstructionsDirty = true;
	end
	
	-- self:ChangeListValues(true);
end

function EffectManager:ChangeListValues(keepIndex)
	if keepIndex==nil then
		keepIndex=false;
	end
	local currentIndex = self.listBox:GetSelectedIndex();
	self.ToDeletedButton:SetVisible(verbose>=2);
	
	local c = self.listBox:GetItemCount();
	for i=1,c do
		local item = self.listBox:GetItem(1);
		self.listBox:RemoveItem(item);
	end
	
	local instructions = { };
	if self.activeList==0 then
		return;
	elseif self.activeList==1 then
		instructions=self.uncategorizedEffectInstructions;
	elseif self.activeList==2 then
		instructions=self.activeEffectInstructions;
	elseif self.activeList==3 then
		instructions=self.inactiveEffectInstructions;
	end

	local fontColor=Turbine.UI.Color(1,.9,.5);
	local backColor=Turbine.UI.Color(.3,.3,.3);
	local fontFace=Turbine.UI.Lotro.Font.Verdana14;

	for i=1,#instructions do
		local itemLabel=OozStuff.OozUI.IconLabel();
		itemLabel:SetSize(self.listBox:GetWidth(),self.fsize);
		itemLabel:SetFont(fontFace);
		itemLabel:SetForeColor(fontColor);
		itemLabel:SetBackColor(backColor);
		local text = instructions[i]:GetName(self.language);
		if text=="" then
			text = "?";
		end
		if instructions[i]:GetIcon() ~= nil then
			itemLabel:SetBackground(instructions[i]:GetIcon());
		end
		itemLabel:SetText(text);
		
		self.listBox:AddItem(itemLabel);
	end
	
	if self.listBox:GetItemCount()>0 then
		if keepIndex then
			if self.listBox:GetItemCount()>=currentIndex then
				self.listBox:SetSelectedIndex(currentIndex);
			else
				self.listBox:SetSelectedIndex(self.listBox:GetItemCount());
			end
		elseif self.activeList==1 then
			self.listBox:SetSelectedIndex(self.listBox:GetItemCount());
		else
			self.listBox:SetSelectedIndex(1);
		end
		self.ToIgnoredButton:SetEnabled(self.activeList==1 or self.activeList==2);
		self.ToActiveButton:SetEnabled(self.activeList==1 or self.activeList==3);
		self.ddEffectPriority:SetVisible(true);
		self.ddEffectCureAction:SetVisible(true);
		self.ddEffectMoveAction:SetVisible(true);
		self.cbEffectYellAction:SetVisible(true);
		self.tEffectComment:SetVisible(true);
	else
		self.lEffectName:SetText("");
		self.lEffectDesc:SetText("");
		self.lEffectInfo:SetText("");
		self.ToIgnoredButton:SetEnabled(false);
		self.ToActiveButton:SetEnabled(false);
		self.ddEffectPriority:SetVisible(false);
		self.ddEffectCureAction:SetVisible(false);
		self.ddEffectMoveAction:SetVisible(false);
		self.cbEffectYellAction:SetVisible(false);
		self.tEffectComment:SetVisible(false);
	end
end

function EffectManager:Closed(sender, args)
	self:SaveData();
end


function EffectManager:AddInitialEffectInstructions( instruction )
	self.activeEffectInstructions[#self.activeEffectInstructions+1] = instruction;
end

function EffectManager:GetUnknownEffectCount()
	return #self.uncategorizedEffectInstructions;
end

function EffectManager:IsNotKnown( effect )
	local effectName= effect:GetName();
	local effectIcon= effect:GetIcon();
	for i=1,#self.uncategorizedEffectInstructions do
		if self.uncategorizedEffectInstructions[i]:GetName(self.language)==effectName then
			return false; -- Already known
		elseif self.uncategorizedEffectInstructions[i]:GetName(self.language)=="" and
				self.uncategorizedEffectInstructions[i]:GetIcon()==effectIcon then
			self.uncategorizedEffectInstructions[i]:SetName(self.language, effect:GetName());
			self.uncategorizedEffectInstructions[i]:SetDescription(self.language, effect:GetDescription());
			self.uncategorizedEffectInstructionsDirty = true;
			if self.activeList==1 then
				self:ChangeListValues(true);
			end
			return false; -- Already known
		end
	end
	for i=1,#self.inactiveEffectInstructions do
		if self.inactiveEffectInstructions[i]:GetName(self.language)==effectName then
			return false; -- Already known
		elseif self.inactiveEffectInstructions[i]:GetName(self.language)=="" and
				self.inactiveEffectInstructions[i]:GetIcon()==effectIcon then
			self.inactiveEffectInstructions[i]:SetName(self.language, effect:GetName());
			self.inactiveEffectInstructions[i]:SetDescription(self.language, effect:GetDescription());
			self.inactiveEffectInstructionsDirty = true;
			if self.activeList==2 then
				self:ChangeListValues(true);
			end
			return false; -- Already known
		end
	end
	for i=1,#self.activeEffectInstructions do
		if self.activeEffectInstructions[i]:GetName(self.language)==effectName then
			return false; -- Already known
		elseif self.activeEffectInstructions[i]:GetName(self.language)=="" and
				self.activeEffectInstructions[i]:GetIcon()==effectIcon then
			self.activeEffectInstructions[i]:SetName(self.language, effect:GetName());
			self.activeEffectInstructions[i]:SetDescription(self.language, effect:GetDescription());
			self.activeEffectInstructionsDirty = true;
			if self.activeList==3 then
				self:ChangeListValues(true);
			end
			return false; -- Already known
		end
	end
	return true;
end

function EffectManager:AddNewEffectInstructions( instruction )
	self.uncategorizedEffectInstructions[#self.uncategorizedEffectInstructions+1] = instruction;
	self.uncategorizedEffectInstructionsDirty= true;
	if self.activeList==1 then
		local index = self.listBox:GetSelectedIndex();
		self:ChangeListValues(true);
		if #self.uncategorizedEffectInstructions==1 then
			index=1;
		end
		self.listBox:SetSelectedIndex(index);
	end
end

function EffectManager:GetActiveEffectInstruction( effect )
	for i=1, #self.activeEffectInstructions do
		if self.activeEffectInstructions[i]:GetName(self.language) == effect:GetName() then
			return self.activeEffectInstructions[i]; -- found it!
		elseif self.activeEffectInstructions[i]:GetName(self.language)=="" and
				self.activeEffectInstructions[i]:GetIcon() == effect:GetIcon() then
			self.activeEffectInstructions[i]:SetName(self.language, effect:GetName());
			self.activeEffectInstructions[i]:SetDescription(self.language, effect:GetDescription());
			self.activeEffectInstructionsDirty = true;
			if self.activeList==1 then
				self:ChangeListValues(true);
			end
			return self.activeEffectInstructions[i]; -- found it!
		end
	end
	return nil;
end

-- import support: merge uncategorized effects into active and ignored effects
function EffectManager:MergeUncategorizedEffects()
	local effectName;
	local effectIcon;
	for j=1,#self.uncategorizedEffectInstructions do
		effectName = self.uncategorizedEffectInstructions[j]:GetName(self.language);
		effectIcon = self.uncategorizedEffectInstructions[j]:GetIcon();

		for i=1,#self.inactiveEffectInstructions do
			if self.inactiveEffectInstructions[i]:GetName(self.language)==effectName then
				effectName = "KNOWN_EFFECT"; -- Already known
			elseif self.inactiveEffectInstructions[i]:GetName(self.language)=="" and
					self.inactiveEffectInstructions[i]:GetIcon()==effectIcon then
				self.inactiveEffectInstructions[i]:SetName(self.language, effect:GetName());
				self.inactiveEffectInstructions[i]:SetDescription(self.language, effect:GetDescription());
				self.inactiveEffectInstructionsDirty = true;
				--if self.activeList==2 then
				--	self:ChangeListValues(true);
				--end
				effectName = "KNOWN_EFFECT"; -- Already known
			end
		end
		for i=1,#self.activeEffectInstructions do
			if self.activeEffectInstructions[i]:GetName(self.language)==effectName then
				effectName = "KNOWN_EFFECT"; -- Already known
			elseif self.activeEffectInstructions[i]:GetName(self.language)=="" and
					self.activeEffectInstructions[i]:GetIcon()==effectIcon then
				self.activeEffectInstructions[i]:SetName(self.language, effect:GetName());
				self.activeEffectInstructions[i]:SetDescription(self.language, effect:GetDescription());
				self.activeEffectInstructionsDirty = true;
				--if self.activeList==3 then
				--	self:ChangeListValues(true);
				--end
				effectName = "KNOWN_EFFECT"; -- Already known
			end
		end

		self.uncategorizedEffectInstructions[j]:SetName(self.language, "KNOWN_EFFECT");	
	end
	
	local mergeCount = 0;
	
	for j=#self.uncategorizedEffectInstructions,1,-1 do
		if self.uncategorizedEffectInstructions[j]:GetName(self.language)=="KNOWN_EFFECT" then
			table.remove(self.uncategorizedEffectInstructions,j);
			mergeCount = mergeCount + 1;
		end
	end

	if mergeCount > 0 then
		self.uncategorizedEffectInstructionsDirty = true;
		Turbine.Shell.WriteLine("Import: removed "..mergeCount.." already known effects.");
	end
	
end

function EffectManager:LoadData()
	-- try to load saved configuration
	local settings={};
	local version="";
	settings=PatchDataLoad( Turbine.DataScope.Account, "CarebearEMSettings");
	if (settings~=nil) and tostring(type(settings))=="table" then
		for index=1,#settings do
			if settings[index]~=nil and #(settings[index])==2 then
				if settings[index][1]=="Version" then
					version=settings[index][2];
				end
			end
		end
		if Plugins["Carebear"]:GetVersion()==version then
			for index=1,#settings do
				if settings[index]~=nil and #(settings[index])==2 then
					if settings[index][1]=="Top" then
						self:SetTop(settings[index][2]);
					elseif settings[index][1]=="Left" then
						self:SetLeft(settings[index][2]);
					elseif settings[index][1]=="Threshold" then
						mainWindow.effectDisplay:SetPriorityThreshold(settings[index][2]);
						self.ddPriorityThreshold:SetLabel(self.priorityValues[mainWindow.effectDisplay:GetPriorityThreshold()]);
--					elseif settings[index][1]=="Height" then
--						self:SetHeight(settings[index][2]);
--					elseif settings[index][1]=="Width" then
--						self:SetWidth(settings[index][2]);
					end
				end
			end
			-- Turbine.Shell.WriteLine("carebear: em window configuration loaded.");
		end
	end
	
	self:LoadEffectData();
end

function EffectManager:LoadEffectData()
	-- try to load saved configuration
	
	self.activeEffectInstructions=self:Deserialize(PatchDataLoad( Turbine.DataScope.Account, "Carebear_AEffects"));
	if self.activeEffectInstructions~=nil and type(self.activeEffectInstructions)=="table" then
		if verbose>=2 then
			Turbine.Shell.WriteLine("loaded "..#self.activeEffectInstructions.." active effects");
		end
		self.activeEffectInstructionsDirty = false;
	else
		if verbose>=2 then
			if self.activeEffectInstructions~=nil then
				Turbine.Shell.WriteLine("type(self.activeEffectInstructions): "..type(self.activeEffectInstructions));
			end
		end
		Turbine.Shell.WriteLine("initialized by prefined effect list with "..#initialEffectInstructions.." active effects");
		self.activeEffectInstructions = initialEffectInstructions;
	end
	self.activeEffectInstructions = self:Quicksort(self.activeEffectInstructions);
	
	self.inactiveEffectInstructions=self:Deserialize(PatchDataLoad( Turbine.DataScope.Account, "Carebear_IEffects"));
	if self.inactiveEffectInstructions~=nil and type(self.inactiveEffectInstructions)=="table" then
		if verbose>=2 then
			Turbine.Shell.WriteLine("loaded "..#self.inactiveEffectInstructions.." inactive effects");
		end
		self.inactiveEffectInstructionsDirty = false;
	else
		self.inactiveEffectInstructions = { };
	end
	self.inactiveEffectInstructions = self:Quicksort(self.inactiveEffectInstructions);
	
	self.uncategorizedEffectInstructions=self:Deserialize(PatchDataLoad( Turbine.DataScope.Account, "Carebear_UCEffects"));
	if self.uncategorizedEffectInstructions~=nil and type(self.uncategorizedEffectInstructions)=="table" then
		if verbose>=2 then
			Turbine.Shell.WriteLine("loaded "..#self.uncategorizedEffectInstructions.." uncategorized effects");
		end
		self.uncategorizedEffectInstructionsDirty = false;
	else
		self.uncategorizedEffectInstructions = { };
	end
	self.uncategorizedEffectInstructions = self:Quicksort(self.uncategorizedEffectInstructions);
	
	self:MergeUncategorizedEffects();
	
	self:ChangeListValues(false);
end

function EffectManager:Deserialize(loadedEffects)
	local effects = { };
	
	if loadedEffects~=nil and type(loadedEffects)=="table" then
		for i=1,#loadedEffects do
			local actionMove = loadedEffects[i]["actionMove"];
			local actionCure = loadedEffects[i]["actionCure"];
			local nameArray = loadedEffects[i]["nameArray"];
			-- Turbine.Shell.WriteLine("loaded effect "..nameArray[self.language]);
			local actionTypeMask = loadedEffects[i]["actionTypeMask"];
			local icon = loadedEffects[i]["icon"];
			local descriptionArray = loadedEffects[i]["descriptionArray"];
			local curable = loadedEffects[i]["curable"];
			local debuff = loadedEffects[i]["debuff"];
			local dmgType = loadedEffects[i]["dmgType"];
			local comment = loadedEffects[i]["comment"];
			local priority = NEM_P_LOW;
			if loadedEffects[i]["priority"]~=nil then
				priority = loadedEffects[i]["priority"];
			end
			
			-- Release 2.3: Value changed of NEM_M_FLAG_YELL
			if dmgType>=1024 and dmgType<NEM_M_FLAG_YELL then
				dmgType = dmgType - 1024 + NEM_M_FLAG_YELL;
			end
			if actionTypeMask>=1024 and actionTypeMask<NEM_M_FLAG_YELL then
				actionTypeMask = actionTypeMask - 1024 + NEM_M_FLAG_YELL;
			end
			
			effects[i]=NamedEffectInstructions(nameArray, actionCure, actionMove, actionTypeMask, 
					icon, descriptionArray, curable, debuff, dmgType, comment, priority);
		end
		return effects;
	else
		return nil;
	end
end

function EffectManager:SaveData()
	local settings={};
	settings[#settings+1]={"Top",self:GetTop()};
	settings[#settings+1]={"Left",self:GetLeft()};
	settings[#settings+1]={"Version",Plugins["Carebear"]:GetVersion()};
	settings[#settings+1]={"Threshold", mainWindow.effectDisplay:GetPriorityThreshold(i);};

	PatchDataSave( Turbine.DataScope.Account, "CarebearEMSettings", settings);
	-- Turbine.Shell.WriteLine("carebear: em window configuration saved.");
	self:SaveEffectData();
end

function EffectManager:SaveEffectData()
	if self.activeEffectInstructionsDirty and self.activeEffectInstructions~=nil and type(self.activeEffectInstructions)=="table" then
		PatchDataSave( Turbine.DataScope.Account, "Carebear_AEffects", self.activeEffectInstructions);
		if verbose>=2 then
			Turbine.Shell.WriteLine("saved "..#self.activeEffectInstructions.." active effects");
		end
	end
	if self.inactiveEffectInstructionsDirty and self.inactiveEffectInstructions~=nil and type(self.inactiveEffectInstructions)=="table" then
		PatchDataSave( Turbine.DataScope.Account, "Carebear_IEffects", self.inactiveEffectInstructions);
		if verbose>=2 then
			Turbine.Shell.WriteLine("saved "..#self.inactiveEffectInstructions.." inactive effects");
		end
	end
	if self.uncategorizedEffectInstructionsDirty and self.uncategorizedEffectInstructions~=nil and type(self.uncategorizedEffectInstructions)=="table" then
		PatchDataSave( Turbine.DataScope.Account, "Carebear_UCEffects", self.uncategorizedEffectInstructions);
		if verbose>=2 then
			Turbine.Shell.WriteLine("saved "..#self.uncategorizedEffectInstructions.." uncategorized effects");
		end
	end
end

function EffectManager:QuicksortLessOrEqual( t1, t2 )
	return (lexicographic(t1:GetName(self.language))<=lexicographic(t2:GetName(self.language)));
end

function EffectManager:Quicksort( t, start, endi )
	start = start or 1;
	endi  = endi or #t;
	
	--partition w.r.t. first element
	if ( endi - start < 2 ) then
		return t;
	end 

	local pivot = start;

	for i = start + 1, endi do
		if ( self:QuicksortLessOrEqual(t[i],t[pivot]) ) then
			local temp = t[pivot + 1];
			t[pivot + 1] = t[pivot];
			
			if(i == pivot + 1) then
				t[pivot] = temp;
			else
				t[pivot] = t[i];
				t[i] = temp;
			end
			
			pivot = pivot + 1;
		end
	end 
	
	t = self:Quicksort( t, start, pivot - 1 );
	return self:Quicksort( t, pivot + 1, endi );
end
