-- Import Turbine Packages. http://www.lotrointerface.com/wiki/LotRO_API_Reference#Packages
import "Turbine";
import "Turbine.Gameplay";
import "Turbine.UI";
import "Turbine.UI.Lotro";

-- This is the Load/Save patch which accounts for European formats, written by Vindar
import "OozStuff.Carebear.VindarPatch";

-- load resource strings - this MUST occur before any windows that use strings get loaded or there will be no defaults
import "OozStuff.Carebear.Resource";

import "OozStuff.Carebear.EffectDisplay";
import "OozStuff.Carebear.NamedEffectInstructions";
import "OozStuff.Carebear.CategoryQuickSlots";
import "OozStuff.Carebear.InitialConfiguration";
import "OozStuff.Carebear.EffectManager";
import "OozStuff.Carebear.Rainbowize";
import "OozStuff.OozUI.ApplicationSymbol";
import "OozStuff.OozUI.LightWindow";

import "OozStuff.Carebear.I18N"; -- Internationalization


verbose = 1; -- 0,1,2
language=GetClientLanguage();

trimColor=Turbine.UI.Color(.9,.2,.9);
fontColor=Turbine.UI.Color(1,.9,.5);
backColor=Turbine.UI.Color(.05,.05,.05);

fontFace=Turbine.UI.Lotro.Font.Verdana16;
alertFontFace=Turbine.UI.Lotro.Font.TrajanProBold22;
bigFontFace=Turbine.UI.Lotro.Font.TrajanProBold36;


function UnloadPlugin()
	SaveData();
	Turbine.Shell.WriteLine(Rainbowize("Carebear")..": bye");
end

function SaveData()
	mainWindow:SaveData();
	mainWindow.effectManager:SaveData();
end


MainWindow = class( OozStuff.OozUI.LightWindow );

function MainWindow:Constructor()
	OozStuff.OozUI.LightWindow.Constructor( self );
	self.effectSize = 160;
	local initWidth=self.effectSize+36*5;
	local titleOffset=16;
	local initHeight=titleOffset+self.effectSize;
	self:SetSize(initWidth,initHeight);
	self:SetTitleColor(Turbine.UI.Color(0,0,.25));
	self:SetDecorationColor(Turbine.UI.Color( .7, .7, .5 ));
	self:SetBackColor(Turbine.UI.Color(1,0,0,0));
	self:SetBackColorBlendMode(2);
	self.Loaded=false;
	self.ShowIconWhenMinimized=true;
	self.initialFlags = { true, true, true, true, true};

	self.effectManager=EffectManager(language);
	
	-- load the settings and set the window position if possible, otherwise default to center
	local initLeft=(Turbine.UI.Display:GetWidth() - initWidth) / 2 - initWidth/2 - 60;
	local initTop=(Turbine.UI.Display:GetHeight() - initHeight) / 2;

	-- account for changed resolution
	if initLeft>(Turbine.UI.Display.GetWidth()-initWidth) then initLeft=Turbine.UI.Display.GetWidth()-initWidth end;
	if initLeft<0 then initLeft=0 end;
	if initWidth>(Turbine.UI.Display.GetWidth()-initLeft) then initWidth=Turbine.UI.Display.GetWidth()-initLeft end;
	if initTop>(Turbine.UI.Display.GetHeight()-initHeight) then initTop=Turbine.UI.Display.GetHeight()-initHeight end;
	if initTop<0 then initTop=0 end;
	if initHeight>(Turbine.UI.Display.GetHeight()-initTop) then initTop=Turbine.UI.Display.GetHeight()-initHeight end;

	self:SetOpacity(1);
	self:SetPosition( initLeft, initTop );
	self:SetVisible(false);
	self:SetText(Resource[language][2]);

	self.symbolClickCallback = function()
		self.symbol:SetActive(false); 
		self.symbol:SetText("");
		if self:IsVisible() and self.effectManager:IsVisible() then
			self:SetVisible(false);
			self:SetDecorationVisible(false);
			self:SetBackColor(Turbine.UI.Color(0,0,0,0));
			self:SetBackColorBlendMode(3);
			self.effectManager:SaveData();
			self.effectManager:SetVisible(false);
		else
			self:SetDecorationVisible(true);
			self:SetBackColor(Turbine.UI.Color(1,0,0,0));
			self:SetBackColorBlendMode(2);
			self.qs:setVisible(self.initialFlags);
			self.effectManager:SetVisible(true);
			self:SetVisible(true);
		end
	end
	self.symbol = OozStuff.OozUI.ApplicationSymbol("OozStuff/Carebear/CarebearSymbol.tga", 48, 48, self.symbolClickCallback);
	self.symbol:SetActiveAnimation({
		"OozStuff/Carebear/CarebearSymbolActive1.tga",
		"OozStuff/Carebear/CarebearSymbolActive2.tga",
		"OozStuff/Carebear/CarebearSymbolActive3.tga"
	});
	self.symbol:SetFramesPerActiveImage(4); -- every x-th frame continue animation
	self.symbolActiveStartTime = Turbine.Engine.GetGameTime();
	self.symbolActiveCallback = function()
		if Turbine.Engine.GetGameTime()-900>symbolActiveStartTime then
			self.symbol:SetActive(false); -- Turn off after 15 Minutes
			self.symbol:SetText("");
		end
	end
	self.symbol:SetActiveTickCallback(symbolActiveCallback);
	self:LoadData();

	-------------------------
	-- Main Window Content --
	-------------------------
	
	self.effectDisplay=EffectDisplay();
	self.effectDisplay:SetParent( self );
	self.effectDisplay:SetSize( self.effectSize, self.effectSize );
	self.effectDisplay:SetPosition( 0, titleOffset );
	self.effectDisplay:SetVisible(false);
	self.effectDisplay:SetMouseVisible(false);
	
	self.TimeTextArea=Turbine.UI.Label();
	self.TimeTextArea:SetParent(self);
	self.TimeTextArea:SetPosition(self.effectDisplay:GetLeft()+self.effectSize,self.effectDisplay:GetTop()+self.effectSize/2-13);
	self.TimeTextArea:SetSize(self.effectSize,self.effectSize);
	self.TimeTextArea:SetForeColor(Turbine.UI.Color(1,.2,.2));
	self.TimeTextArea:SetFontStyle( Turbine.UI.FontStyle.Outline );
	self.TimeTextArea:SetOutlineColor( Turbine.UI.Color( 0,0,0 ) );
	self.TimeTextArea:SetFont(bigFontFace);
	self.TimeTextArea:SetText("");
	self.TimeTextArea:SetZOrder(2);
	self.TimeTextArea:SetMouseVisible(false);
	
	self.qs=CategoryQuickSlots(language);
	self.qs:SetParent(self);
	self.qs:SetPosition(self.effectDisplay:GetLeft()+self.effectSize,self.effectDisplay:GetTop());
	self.qs:SetMouseVisible(false);

	self.AlertTextArea=Turbine.UI.Label();
	self.AlertTextArea:SetParent(self);
	self.AlertTextArea:SetSize(self.effectSize,self.effectSize/2);
	self.AlertTextArea:SetPosition(self.effectDisplay:GetLeft()+self.effectSize,self.effectDisplay:GetTop()+self.effectSize/2+15);
	self.AlertTextArea:SetForeColor(Turbine.UI.Color(1,.8,.8));
	self.AlertTextArea:SetFontStyle( Turbine.UI.FontStyle.Outline );
	self.AlertTextArea:SetOutlineColor( Turbine.UI.Color( 0,0,0 ) );
	self.AlertTextArea:SetFont(alertFontFace);
	self.AlertTextArea:SetText("");
	self.AlertTextArea:SetMouseVisible(false);
	
	self.qs:setVisible(self.initialFlags);
	
	--------------------
	-- Shell Handling --
	--------------------
	
	self.RemoveShellCommands=function(sender,args)
		Turbine.Shell.RemoveCommand(self.shellCmd)
	end
	self.shellCmd=Turbine.ShellCommand();
	local numberOfCommandsRegistered = Turbine.Shell.AddCommand("Carebear", self.shellCmd);

	self.shellCmd.Execute = function(sender, cmd, args)
		if (string.lower(cmd) == "carebear") then
			if (string.lower(args)=="show") then
				mainWindow:SetVisible(true);
				self.qs:setVisible(self.initialFlags);
			elseif (string.lower(args)=="hide") then
				mainWindow:SetVisible(false);
			elseif (string.lower(args)=="setup") then
				self.symbol:SetActive(false);
				self.symbol:SetText("");
				self.effectManager:SetVisible(true);
			elseif (string.lower(args)=="toggle") then
				mainWindow:SetVisible(not mainWindow:IsVisible());
			elseif (string.lower(args)=="debug") then
				Turbine.Shell.WriteLine("carebear:  output set to debug");
				verbose = 2;
				self.effectManager:ChangeListValues(true);
			elseif (string.lower(args)=="verbose") then
				Turbine.Shell.WriteLine("carebear:  output set to verbose");
				verbose = 1;
				self.effectManager:ChangeListValues(true);
			elseif (string.lower(args)=="silent") then
				Turbine.Shell.WriteLine("carebear:  output set to silent");
				verbose = 0;
				self.effectManager:ChangeListValues(true);
			else
				Turbine.Shell.WriteLine("usage:");
				Turbine.Shell.WriteLine("  /carebear show");
				Turbine.Shell.WriteLine("       Displays the carebear bar.");
				Turbine.Shell.WriteLine("  /carebear hide");
				Turbine.Shell.WriteLine("       Hides the carebear bar.");
				Turbine.Shell.WriteLine("  /carebear toggle");
				Turbine.Shell.WriteLine("       Toggles the carebear bar (shows it if it was hidden, hides it if it was showing).");
				Turbine.Shell.WriteLine("  /carebear [silent|verbose|debug]");
				Turbine.Shell.WriteLine("       No, verbose or very verbose output on effects.");
			end
		end
	end
	self.KeyDown=function(sender, args)
--Turbine.Shell.WriteLine("KeyDown Action="..args.Action..", .Alt="..tostring(args.Alt)..", .Control="..tostring(args.Control)..", .Shift="..tostring(args.Shift))
		if ( args.Action == Turbine.UI.Lotro.Action.Escape ) then
			mainWindow:SetVisible( false );
		end
		if ( args.Action == 268435635 ) then -- toggle HUD
			mainWindow:SetVisible( false );
		end
	end

	self.shellCmd.GetHelp = function(sender, cmd)
		return("usage:\n  /carebear show\n       Displays the Carebear.\n  /carebear hide\n       Hides Carebear.\n  /carebear toggle\n       Toggles the Carebear (shows it if it was hidden, hides it if it was showing).");
	end

	self.shellCmd.GetShortHelp = function(sender, cmd)
		return("Carebear - usage: /carebear show | hide | toggle");
	end

	self.GetShowIconWhenMinimized=function()
		return self.ShowIconWhenMinimized;
	end

	self.SetTrimColor=function()
	end
	self.SetBackColor=function()
	end
	self.SetFontColor=function()
	end
	self.SetLanguage=function()
		self:SetText(Resource[language][2]);
		self.PreviewCaption:SetText(Resource[language][2]..":");
	end

	self.Update=function( sender, args )
		if (Plugins["Carebear"] ~= nil) and (not self.Loaded) then
			self.Loaded=true;
			Turbine.Shell.WriteLine(Rainbowize("Carebear "..Plugins["Carebear"]:GetVersion()));
			Plugins["Carebear"].Unload = function(self,sender,args)
				UnloadPlugin();
			end
		elseif not self.Initialized then
			local m = Turbine.Gameplay.LocalPlayer.GetInstance():GetMorale();
			local mm = Turbine.Gameplay.LocalPlayer.GetInstance():GetMaxMorale();
			local bmm = Turbine.Gameplay.LocalPlayer.GetInstance():GetBaseMaxMorale();
			if m == mm and m==bmm then
				self.qs:FitPotions();
				if mainWindow.qs:PotionsFitted() then
					Turbine.Shell.WriteLine("Carebear: potions ready.");
				else
					mainWindow:SetVisible(true);
					Turbine.Shell.WriteLine("Carebear: buy more potions!");
				end
				self.Initialized=true;
			end
		end
	end
end

function MainWindow:HookIntoPlayer()

	self.player = Turbine.Gameplay.LocalPlayer.GetInstance();
	local effects = self.player:GetEffects();

	for i = 1, effects:GetCount() do
		self:AddEffect( i );
	end
	
	effects.EffectAdded = function( sender, args )
		self:AddEffect( args.Index );
	end
	
	effects.EffectRemoved = function( sender, args )
		self:RemoveEffect( args.Effect );
	end
	
	effects.EffectsCleared = function( sender, args )
		self.effectsList:Clear();
	end
	
	self:SetWantsUpdates(true);
end

function MainWindow:Closing(sender, args)
	self:SaveData();
end

function MainWindow:ConfigEdited()
	self:SaveData();
end

function MainWindow:AddEffect( effectIndex )
	local effect = self.player:GetEffects():Get( effectIndex );

	local currentEffectInstruction = self.effectManager:GetActiveEffectInstruction(effect);
	if currentEffectInstruction ~= nil and self.effectDisplay:CheckPriority(currentEffectInstruction) then
		self:SetVisible(true);
		self:SetDecorationVisible(false);
		self:SetBackColor(Turbine.UI.Color(0,0,0,0));
		self:SetBackColorBlendMode(3);
		if verbose>=2 then
			Turbine.Shell.WriteLine("carebear window activated.");
		end
		local flags = { 
			currentEffectInstruction:HasFearAction(),
			currentEffectInstruction:HasWoundAction(),
			currentEffectInstruction:HasDiseaseAction(),
			currentEffectInstruction:HasPoisonAction(),
			currentEffectInstruction:HasYellAction()
		};
		self.qs:setVisible(flags);
		self:UpdateAlert(currentEffectInstruction);

		self.effectDisplay:SetEffect( effect, self.TimeTextArea, self.AlertTextArea, currentEffectInstruction,
			currentEffectInstruction:GetActionCure()==NEM_A_CURE_EXPIRES );
	elseif self.effectManager:IsNotKnown(effect) then
		local effectName = {"", "", ""};
		effectName[language]=effect:GetName();
		local description = {"", "", ""};
		description[language]=effect:GetDescription();
		local now = Turbine.Engine.GetDate().Year.."/"..Turbine.Engine.GetDate().Month.."/"..Turbine.Engine.GetDate().Day;
		now=now.." "..Turbine.Engine.GetDate().Hour..":";
		if Turbine.Engine.GetDate().Minute < 10 then
			now=now.."0";
		end
		now=now..Turbine.Engine.GetDate().Minute..":";
		if Turbine.Engine.GetDate().Second < 10 then
			now=now.."0";
		end
		now=now..Turbine.Engine.GetDate().Second;
		local comment = "["..Turbine.Gameplay.LocalPlayer:GetInstance():GetName().." "..now.."] "..Resource[language][15]..".";
		local newEffectInstruction = NamedEffectInstructions(
			effectName, NEM_A_CURE_NOW, NEM_T_MOVE_WHATEVER, 
			effect:GetCategory(), effect:GetIcon(), description,
			effect:IsCurable(), effect:IsDebuff(), effect:GetCategory(), comment, NEM_P_LOW);
			
		self.effectManager:AddNewEffectInstructions(newEffectInstruction);
		self.symbolActiveStartTime = Turbine.Engine.GetGameTime();
		self.symbol:SetText(""..self.effectManager:GetUnknownEffectCount());
		self.symbol:SetActive(true);

		if verbose>=2 or (verbose>=1 and effect:GetCategory()>0) then
			local text = "carebear "..Resource[language][15]..": "..effect:GetName();
			if verbose>=2  then
				text = text..": "..effect:GetDescription();
			end
			Turbine.Shell.WriteLine(text);
		end
		
	end
end


function MainWindow:UpdateAlert( instruction )
	if instruction:GetActionCure()==NEM_A_CURE_DISTANCE then 
		-- This warning has priority
		self.AlertTextArea:SetText(Resource[language][5]); 
	else
		if instruction:GetActionCure()==NEM_A_CURE_EXPIRES then 
			self.AlertTextArea:SetText(Resource[language][58]); 
		elseif instruction:GetActionMove()==NEM_T_MOVE_NOT then
			self.AlertTextArea:SetText(Resource[language][4]);
		elseif instruction:GetActionMove()==NEM_T_MOVE_NOW then
			self.AlertTextArea:SetText(Resource[language][6]);
		elseif instruction:GetActionMove()==NEM_T_MOVE_AWAY then
			self.AlertTextArea:SetText(Resource[language][7]);
		elseif instruction:GetActionMove()==NEM_T_MOVE_OUT then
			self.AlertTextArea:SetText(Resource[language][8]);
		else
			self.AlertTextArea:SetText("");
		end
	end
end

function MainWindow:RemoveEffect( effect )
	if self.effectDisplay:RemoveEffect( effect ) then
		-- Debug: Turbine.Shell.WriteLine("carebear: effect removed.");
		self:SetVisible(false);
	end
end

function MainWindow:LoadData()
	-- try to load saved configuration
	local settings={};
	local version="";
	settings=PatchDataLoad( Turbine.DataScope.Account, "CarebearSettings");
	if (settings~=nil) and tostring(type(settings))=="table" then
		for index=1,#settings do
			if settings[index]~=nil and #(settings[index])==2 then
				if settings[index][1]=="Version" then
					version=settings[index][2];
				end
			end
		end
		-- if not Plugins["Carebear"]:GetVersion()==version then
		--	Turbine.Shell.WriteLine("carebear: old window configuration ignored.");
		-- else
			for index=1,#settings do
				if settings[index]~=nil and #(settings[index])==2 then
					if settings[index][1]=="Top" then
						self:SetTop(settings[index][2]);
					elseif settings[index][1]=="Left" then
						self:SetLeft(settings[index][2]);
					elseif settings[index][1]=="Symbol.Top" then
						self.symbol:SetTop(settings[index][2]);
					elseif settings[index][1]=="Symbol.Left" then
						self.symbol:SetLeft(settings[index][2]);
					elseif settings[index][1]=="WindowProperties" then
						self:SetConfig(settings[index][2]);
					end
				end
			end
			-- Turbine.Shell.WriteLine("carebear: main window configuration loaded.");
		-- end
	end
end


function MainWindow:SaveData()
	local settings={};
	settings[#settings+1]={"Version",Plugins["Carebear"]:GetVersion()};
	settings[#settings+1]={"Top",self:GetTop()};
	settings[#settings+1]={"Left",self:GetLeft()};
	settings[#settings+1]={"Symbol.Top",self.symbol:GetTop()};
	settings[#settings+1]={"Symbol.Left",self.symbol:GetLeft()};
	settings[#settings+1]={"WindowProperties",self:GetConfig()};
	PatchDataSave( Turbine.DataScope.Account, "CarebearSettings", settings);
	-- Turbine.Shell.WriteLine("carebear: main window configuration saved.");
end

mainWindow=MainWindow();
mainWindow:SetZOrder(mainWindow:GetZOrder()+1);
mainWindow:SetBackColor(Turbine.UI.Color(0,0,0,0));
mainWindow:SetBackColorBlendMode(3);
mainWindow.effectManager:SetZOrder(mainWindow:GetZOrder()+1);
mainWindow.effectManager:LoadData();
mainWindow:HookIntoPlayer();
