import "Turbine";
import "Turbine.UI";
import "Turbine.UI.Lotro";

import "OozStuff.OozUI.LightSlider";

-- Simple Window with
--  close button
--  title bar (dragging)

LightWindow = class( Turbine.UI.Window );

function LightWindow:Constructor( )
	Turbine.UI.Window.Constructor( self );
	self:SetBackColor(Turbine.UI.Color(1,0,0,0));	
	self.drag = false;

	self.opacity = 1.0;
	self.clickthrough = true;
	
	self:SetMouseVisible(not clickthrough);
	
	self.closeable = true;
	self.Configuarable = true;

	self.titleButton = Turbine.UI.Button();
	self.titleButton:SetParent(self);
	self.titleButton:SetSize( 144, 16 );
	self.titleButton:SetPosition(0, 0);
	self.titleButton:SetFontStyle( Turbine.UI.FontStyle.Outline );
	self.titleButton.MouseDown = function( sender, args )
			self.drag = true;
			self.mx0 = args.X;
			self.my0 = args.Y;
	end
	self.titleButton.MouseUp = function( sender, args )
		self.drag = false;
	end
	self.titleButton.MouseMove = function( sender, args )
		if self.drag then
			local dx = args.X - self.mx0;
			local dy = args.Y - self.my0;
			-- Turbine.Shell.WriteLine("dragged "..dx.." "..dy);
			self.thresholdTrigger = true;
			local x = self:GetLeft()+dx;
			local y = self:GetTop()+dy;
			if x<0 then x=0 end;
			if y<0 then y=0 end;
			local sh = Turbine.UI.Display.GetHeight()-self:GetHeight()-1;
			local sw = Turbine.UI.Display.GetWidth()-self:GetWidth()-1;
			if x>sw then x=sw end;
			if y>sh then y=sh end;
			self:SetLeft(x);
			self:SetTop(y);
		end
	end

	self.contextMenu = Turbine.UI.ContextMenu();
	self:BuildContextMenu();
	
	self.menuButton = Turbine.UI.Button();
	self.menuButton:SetParent(self);
	self.menuButton:SetSize( 14, 14 );
	self.menuButton:SetPosition( 1, 1 );
	self.menuButton:SetBackColor(Turbine.UI.Color(.05,.05,.05));
	self.menuButton:SetFont(Turbine.UI.Lotro.Font.Verdana12);
	self.menuButton:SetFontStyle( Turbine.UI.FontStyle.Outline );
	self.menuButton:SetText(""..string.char(12*16+14)..string.char(9*16+14));
	self.menuButton.MouseClick = function( sender, args )
		self.contextMenu:ShowMenuAt(self:GetLeft()+2,self:GetTop()+15);
	end
	self.menuButton.MouseEnter = function( sender, args )
		self.menuButton:SetBackColor(Turbine.UI.Color(.2,.2,.6));
	end
	self.menuButton.MouseLeave = function( sender, args )
		self.menuButton:SetBackColor(Turbine.UI.Color(.05,.05,.05));
	end	
	
	self.closeButton = Turbine.UI.Button();
	self.closeButton:SetParent(self);
	self.closeButton:SetSize( 14, 14 );
	self.closeButton:SetBackColor(Turbine.UI.Color(.05,.05,.05));
	self.closeButton:SetFont(Turbine.UI.Lotro.Font.Verdana12);
	self.closeButton:SetFontStyle( Turbine.UI.FontStyle.Outline );
	self.closeButton:SetText("X");
	self.closeButton.MouseClick = function( sender, args )
		self:SetVisible(false);
		if self.configWindow~=nil then
			self.configWindow:SetVisible(false);
			self.configWindow = nil;
		end
	end
	self.closeButton.MouseEnter = function( sender, args )
		self.closeButton:SetBackColor(Turbine.UI.Color(.2,.2,.6));
	end
	self.closeButton.MouseLeave = function( sender, args )
		self.closeButton:SetBackColor(Turbine.UI.Color(.05,.05,.05));
	end	
	
	self.topLine = Turbine.UI.Label();
	self.topLine:SetParent(self);
	self.bottomLine = Turbine.UI.Label();
	self.bottomLine:SetParent(self);
	self.leftLine = Turbine.UI.Label();
	self.leftLine:SetParent(self);
	self.rightLine = Turbine.UI.Label();
	self.rightLine:SetParent(self);
	self.titleLine = Turbine.UI.Label();
	self.titleLine:SetParent(self);
	self.closeLine = Turbine.UI.Label();
	self.closeLine:SetParent(self);
	self.menuLine = Turbine.UI.Label();
	self.menuLine:SetParent(self);
	
	self:SetDecorationColor(Turbine.UI.Color( 1, 1, .8 ));
	self:SetSize(160,100);
	self:SetTitleColor(Turbine.UI.Color(.05,.05,.05));
end

function LightWindow:SetSize(width, height)
	Turbine.UI.Window.SetSize(self, width, height);
	self.closeButton:SetPosition( self:GetWidth()-15, 1 );
	self.titleButton:SetSize( self:GetWidth()-16, 16 );
	self.topLine:SetSize( self:GetWidth(), 1 );
	self.topLine:SetPosition(0, 0);
	self.bottomLine:SetSize( self:GetWidth(), 1 );
	self.bottomLine:SetPosition(0, self:GetHeight()-1);
	self.leftLine:SetSize( 1, self:GetHeight() );
	self.leftLine:SetPosition(0, 0);
	self.rightLine:SetSize( 1, self:GetHeight() );
	self.rightLine:SetPosition(self:GetWidth()-1, 0);
	self.titleLine:SetSize( self:GetWidth(), 1 );
	self.titleLine:SetPosition(0, 15);
	self.closeLine:SetSize( 1, 16 );
	self.closeLine:SetPosition(self:GetWidth()-15, 0);
	self.menuLine:SetSize( 1, 16 );
	self.menuLine:SetPosition(15, 0);
end

function LightWindow:SetHeight(height)
	self:SetSize(self:GetWidth(), height)
end

function LightWindow:SetWidth(width)
	self:SetSize(width, self:GetHeight())
end

function LightWindow:SetText(text)
	self.titleButton:SetText(text);
end

function LightWindow:IsCloseable()
	return self.closeable;
end

function LightWindow:IsConfiguarable()
	return self.Configuarable;
end

function LightWindow:SetCloseable(value)
	self.closeable = value;
end

function LightWindow:SetConfiguarable(value)
	self.Configuarable = value;
end

function LightWindow:SetBorderVisible(visible)
	self.topLine:SetVisible( visible );
	self.bottomLine:SetVisible( visible );
	self.leftLine:SetVisible( visible );
	self.rightLine:SetVisible( visible );
end

function LightWindow:SetDecorationVisible(visible)
	self:SetBorderVisible(visible);
	
	self.titleButton:SetVisible( visible );
	self.closeButton:SetVisible( visible and self:IsCloseable() );
	self.menuButton:SetVisible( visible and self:IsConfiguarable() );
	self.titleLine:SetVisible( visible );
	self.closeLine:SetVisible( visible and self:IsConfiguarable() );
	self.menuLine:SetVisible( visible and self:IsConfiguarable() );
end

function LightWindow:SetDecorationColor(color)
	self.closeButton:SetForeColor( color );
	self.titleButton:SetForeColor( color );
	self.titleButton:SetOutlineColor( Turbine.UI.Color( color.R/5, color.G/5, color.B/5 ) );
	self.topLine:SetBackColor( color );
	self.bottomLine:SetBackColor( color );
	self.leftLine:SetBackColor( color );
	self.rightLine:SetBackColor( color );
	self.titleLine:SetBackColor( color );
	self.closeLine:SetBackColor( color );
	self.menuLine:SetBackColor( color );
end

function LightWindow:SetTitleColor(color)
	self.titleButton:SetBackColor(color);
end

function LightWindow:SetTitleBackground(image)
	self.titleButton:SetBackground(image);
end


function LightWindow:BuildContextMenu()
	local res={};

	-- English
	res[1]={};
	res[1][1]="English";
	res[1][2]="say";
	res[1][3]="Configuration...";
	res[1][4]="Opacity";
	res[1][5]="Click-Through";

	-- French Franaise
	res[2]={};
	res[2][1]="Fran"..string.char(195)..string.char(167).."aise";
	res[2][2]="parler";
	res[2][3]="Configuration...";
	res[2][4]="Opacit"..string.char(195)..string.char(169);
	res[2][5]="Cliquer-Passer";

	-- German Deutsch
	res[3]={};
	res[3][1]="Deutsch";
	res[3][2]="sagen";
	res[3][3]="Einstellungen...";
	res[3][4]="Sichtbarkeit";
	res[3][5]="Durchklicken";

	-- Select language (Lotro Client language, not Operating System language)
	local i18n_language=1; -- Default to english (EN)
	local i18n_cmd = Turbine.Shell.GetCommands();
	for i=1,#i18n_cmd do
		for l=1,3 do
			if i18n_cmd[i]==res[l][2] then
				i18n_language=l;
			end
		end
	end
	
	local menuItems = self.contextMenu:GetItems();
	local configItem = Turbine.UI.MenuItem(res[i18n_language][3]);
	menuItems:Add(configItem);
	configItem.Click = function(sender, args)
		if self.configWindow==nil then
			self:ShowConfigDialog(res[i18n_language]);
		end
	end

end

function LightWindow:ShowConfigDialog(lres)
	self.configWindow=LightWindow();
	self.configWindow:SetDecorationVisible(false);
	self.configWindow:SetBorderVisible(true);
	self.configWindow:SetSize(160,100);
	self.configWindow:SetZOrder(mainWindow:GetZOrder()+2);	
	self.configWindow:SetPosition(
		self:GetLeft()+5,
		self:GetTop()+5);
		
    self.lcOpacity = Turbine.UI.Label();
	self.lcOpacity:SetParent(self.configWindow);
	self.lcOpacity:SetText(lres[4]..":");
	self.lcOpacity:SetForeColor(Turbine.UI.Color(.7,.7,.5));
	self.lcOpacity:SetSize(150,14);
	self.lcOpacity:SetPosition(5, 15);		
		
	local slider = OozStuff.OozUI.LightSlider();
	slider:SetParent(self.configWindow);
	slider:SetWidth(150);
	slider:SetPosition(5, 30);		
	slider:SetMin(0.25);
	slider:SetValue(self.opacity);
	slider.ValueChanged = function()
		self.opacity = slider:GetValue();
	 	self:SetOpacity(self.opacity);
	end
	
    self.cbClickThrough = Turbine.UI.Lotro.CheckBox();
	self.cbClickThrough:SetParent(self.configWindow);
	self.cbClickThrough:SetText(lres[5]);
	self.cbClickThrough:SetWidth(150);
	self.cbClickThrough:SetPosition(5, 55);		
	self.cbClickThrough:SetHeight(18);
	self.cbClickThrough:SetForeColor(Turbine.UI.Color(.7,.7,.5));
	self.cbClickThrough:SetChecked(self.clickthrough);
	self.cbClickThrough.CheckedChanged = function( sender, args )
		self.clickthrough = self.cbClickThrough:IsChecked();
		self:SetMouseVisible(not self.clickthrough);
	end	
	
	local configCloseButton = Turbine.UI.Button();
	configCloseButton:SetParent(self.configWindow);
	configCloseButton:SetSize( 60, 16 );
	configCloseButton:SetPosition( 
		self.configWindow:GetWidth()/2-configCloseButton:GetWidth()/2,
		self.configWindow:GetHeight()-configCloseButton:GetHeight()-3);
	configCloseButton:SetBackColor(Turbine.UI.Color(.35,.35,.35));
	configCloseButton:SetFont(Turbine.UI.Lotro.Font.Verdana14);
	configCloseButton:SetFontStyle( Turbine.UI.FontStyle.Outline );
	configCloseButton:SetText("OK");
	configCloseButton.MouseClick = function( sender, args )
		self.configWindow:SetVisible(false);
		self.configWindow=nil;
		self:ConfigEdited();
	end
	configCloseButton.MouseEnter = function( sender, args )
		configCloseButton:SetBackColor(Turbine.UI.Color(.2,.2,.6));
	end
	configCloseButton.MouseLeave = function( sender, args )
		configCloseButton:SetBackColor(Turbine.UI.Color(.35,.35,.35));
	end	
	
	self.configWindow:SetVisible(true);
end

function LightWindow:GetConfig()
	local settings = {};
	settings[#settings+1]={"opacity",self.opacity};
	settings[#settings+1]={"clickthrough",self.clickthrough};
	return settings;
end

function LightWindow:SetConfig(settings)
	if (settings~=nil) and tostring(type(settings))=="table" then
		for index=1,#settings do
			if settings[index]~=nil and #(settings[index])==2 then
				if settings[index][1]=="opacity" then
					self.opacity = settings[index][2];
					self:SetOpacity(self.opacity);
				elseif settings[index][1]=="clickthrough" then
					self.clickthrough = settings[index][2];
					self:SetMouseVisible(not self.clickthrough);
				end
			end
		end
	end
end

-- Called if config was edited. 
-- use GetConfig() to get the new settings
function LightWindow:ConfigEdited()
end

